@@ -36,6 +36,7 @@ UINTN                    mBootOptionBbsMappingCount = 0;
 extern EFI_DEVICE_PATH EndDevicePath[];
 extern EFI_GUID gEfiLegacyBiosProtocolGuid;
 EFI_GUID gEfiLegacyDevOrderVariableGuid     = { 0xa56074db, 0x65fe, 0x45f7, {0xbd, 0x21, 0x2d, 0x2b, 0xdd, 0x8e, 0x96, 0x52 }};
+static EFI_GUID EfiGlobalVariableGuid = { 0x8BE4DF61, 0x93CA, 0x11D2, { 0xAA, 0x0D, 0x00, 0xE0, 0x98, 0x03, 0x2B, 0x8C }};
 
 /**
 
@@ -276,7 +277,7 @@ BdsFindLegacyBootOptionByDevTypeAndName (
     UnicodeSPrint (BootOption, sizeof (BootOption), L"Boot%04x", (UINTN) BootOrder[Index]);
     BootOptionVar = BdsLibGetVariableAndSize (
                       BootOption,
-                      &gEfiGlobalVariableGuid,
+                      &EfiGlobalVariableGuid,
                       &BootOptionSize
                       );
     if (NULL == BootOptionVar) {
@@ -462,7 +463,7 @@ BdsCreateLegacyBootOption (
 
   Status = refit_call5_wrapper(gRT->SetVariable,
                   BootString,
-                  &gEfiGlobalVariableGuid,
+                  &EfiGlobalVariableGuid,
                   VAR_FLAG,
                   BufferSize,
                   Buffer
@@ -747,7 +748,7 @@ BdsAddNonExistingLegacyBootOptions (
 
   BootOrder = BdsLibGetVariableAndSize (
                 L"BootOrder",
-                &gEfiGlobalVariableGuid,
+                &EfiGlobalVariableGuid,
                 &BootOrderSize
                 );
   if (BootOrder == NULL) {
@@ -817,13 +818,13 @@ BdsAddNonExistingLegacyBootOptions (
   if (BootOrderSize > 0) {
     Status = refit_call5_wrapper(gRT->SetVariable,
                     L"BootOrder",
-                    &gEfiGlobalVariableGuid,
+                    &EfiGlobalVariableGuid,
                     VAR_FLAG,
                     BootOrderSize,
                     BootOrder
                     );
   } else {
-    EfiLibDeleteVariable (L"BootOrder", &gEfiGlobalVariableGuid);
+    EfiLibDeleteVariable (L"BootOrder", &EfiGlobalVariableGuid);
   }
 
   if (BootOrder != NULL) {
@@ -861,7 +862,7 @@ BdsDeleteBootOption (
   Index2Del = 0;
 
   UnicodeSPrint (BootOption, sizeof (BootOption), L"Boot%04x", OptionNumber);
-  Status = EfiLibDeleteVariable (BootOption, &gEfiGlobalVariableGuid);
+  Status = EfiLibDeleteVariable (BootOption, &EfiGlobalVariableGuid);
 
   //
   // adjust boot order array
@@ -940,7 +941,7 @@ BdsDeleteAllInvalidLegacyBootOptions (
 
   BootOrder = BdsLibGetVariableAndSize (
                 L"BootOrder",
-                &gEfiGlobalVariableGuid,
+                &EfiGlobalVariableGuid,
                 &BootOrderSize
                 );
   if (BootOrder == NULL) {
@@ -952,14 +953,14 @@ BdsDeleteAllInvalidLegacyBootOptions (
     UnicodeSPrint (BootOption, sizeof (BootOption), L"Boot%04x", BootOrder[Index]);
     BootOptionVar = BdsLibGetVariableAndSize (
                       BootOption,
-                      &gEfiGlobalVariableGuid,
+                      &EfiGlobalVariableGuid,
                       &BootOptionSize
                       );
     if (NULL == BootOptionVar) {
       BootOptionSize = 0;
       Status = refit_call5_wrapper(gRT->GetVariable,
                       BootOption,
-                      &gEfiGlobalVariableGuid,
+                      &EfiGlobalVariableGuid,
                       NULL,
                       &BootOptionSize,
                       BootOptionVar
@@ -1035,13 +1036,13 @@ BdsDeleteAllInvalidLegacyBootOptions (
   if (BootOrderSize != 0) {
     Status = refit_call5_wrapper(gRT->SetVariable,
                     L"BootOrder",
-                    &gEfiGlobalVariableGuid,
+                    &EfiGlobalVariableGuid,
                     VAR_FLAG,
                     BootOrderSize,
                     BootOrder
                     );
   } else {
-    EfiLibDeleteVariable (L"BootOrder", &gEfiGlobalVariableGuid);
+    EfiLibDeleteVariable (L"BootOrder", &EfiGlobalVariableGuid);
   }
 
   if (BootOrder != NULL) {
