@@ -36,8 +36,9 @@
 /* Changes copyright (c) 2013 Roderick W. Smith */
 
 #include "gptsync.h"
+#include "../include/version.h"
 
-#include "syslinux_mbr.h"
+#include "../include/syslinux_mbr.h"
 #define memcpy(a, b, c) CopyMem(a, b, c)
 
 //
