@@ -62,21 +62,23 @@ EG_IMAGE * BuiltinIcon(IN UINTN Id);
 #define BUILTIN_ICON_FUNC_FIRMWARE         (4)
 #define BUILTIN_ICON_FUNC_CSR_ROTATE       (5)
 #define BUILTIN_ICON_FUNC_HIDDEN           (6)
-#define BUILTIN_ICON_TOOL_SHELL            (7)
-#define BUILTIN_ICON_TOOL_PART             (8)
-#define BUILTIN_ICON_TOOL_RESCUE           (9)
-#define BUILTIN_ICON_TOOL_APPLE_RESCUE     (10)
-#define BUILTIN_ICON_TOOL_WINDOWS_RESCUE   (11)
-#define BUILTIN_ICON_TOOL_MOK_TOOL         (12)
-#define BUILTIN_ICON_TOOL_FWUPDATE         (13)
-#define BUILTIN_ICON_TOOL_MEMTEST          (14)
-#define BUILTIN_ICON_TOOL_NETBOOT          (15)
-#define BUILTIN_ICON_VOL_INTERNAL          (16)
-#define BUILTIN_ICON_VOL_EXTERNAL          (17)
-#define BUILTIN_ICON_VOL_OPTICAL           (18)
-#define BUILTIN_ICON_VOL_NET               (19)
-#define BUILTIN_ICON_MOUSE                 (20)
-#define BUILTIN_ICON_COUNT                 (21)
+#define BUILTIN_ICON_FUNC_INSTALL          (7)
+#define BUILTIN_ICON_FUNC_BOOTORDER        (8)
+#define BUILTIN_ICON_TOOL_SHELL            (9)
+#define BUILTIN_ICON_TOOL_PART             (10)
+#define BUILTIN_ICON_TOOL_RESCUE           (11)
+#define BUILTIN_ICON_TOOL_APPLE_RESCUE     (12)
+#define BUILTIN_ICON_TOOL_WINDOWS_RESCUE   (13)
+#define BUILTIN_ICON_TOOL_MOK_TOOL         (14)
+#define BUILTIN_ICON_TOOL_FWUPDATE         (15)
+#define BUILTIN_ICON_TOOL_MEMTEST          (16)
+#define BUILTIN_ICON_TOOL_NETBOOT          (17)
+#define BUILTIN_ICON_VOL_INTERNAL          (18)
+#define BUILTIN_ICON_VOL_EXTERNAL          (19)
+#define BUILTIN_ICON_VOL_OPTICAL           (20)
+#define BUILTIN_ICON_VOL_NET               (21)
+#define BUILTIN_ICON_MOUSE                 (22)
+#define BUILTIN_ICON_COUNT                 (23)
 
 #endif
 
